@@ -14,22 +14,28 @@
 
 # This file contains the detection logic for external dependencies.
 # Custom logic for several other packages are in separate files.
+
+from __future__ import annotations
 import copy
 import os
+import collections
 import itertools
 import typing as T
 from enum import Enum
 
 from .. import mlog
 from ..compilers import clib_langs
-from ..mesonlib import MachineChoice, MesonException, HoldableObject
+from ..mesonlib import LibType, MachineChoice, MesonException, HoldableObject
 from ..mesonlib import version_compare_many
-from ..interpreterbase import FeatureDeprecated
+#from ..interpreterbase import FeatureDeprecated, FeatureNew
 
 if T.TYPE_CHECKING:
+    from .._typing import ImmutableListProtocol
+    from ..build import StructuredSources
     from ..compilers.compilers import Compiler
     from ..environment import Environment
-    from ..build import BuildTarget
+    from ..interpreterbase import FeatureCheckBase
+    from ..build import BuildTarget, CustomTarget, IncludeDirs
     from ..mesonlib import FileOrString
 
 
@@ -88,10 +94,12 @@ class Dependency(HoldableObject):
         # Raw -L and -l arguments without manual library searching
         # If None, self.link_args will be used
         self.raw_link_args: T.Optional[T.List[str]] = None
-        self.sources: T.List['FileOrString'] = []
-        self.methods = process_method_kw(self.get_methods(), kwargs)
+        self.sources: T.List[T.Union['FileOrString', 'CustomTarget', 'StructuredSources']] = []
         self.include_type = self._process_include_type_kw(kwargs)
         self.ext_deps: T.List[Dependency] = []
+        self.d_features: T.DefaultDict[str, T.List[T.Any]] = collections.defaultdict(list)
+        self.featurechecks: T.List['FeatureCheckBase'] = []
+        self.feature_since: T.Optional[T.Tuple[str, str]] = None
 
     def __repr__(self) -> str:
         return f'<{self.__class__.__name__} {self.name}: {self.is_found}>'
@@ -104,7 +112,7 @@ class Dependency(HoldableObject):
             return mlog.red('NO')
         if not self.version:
             return mlog.green('YES')
-        return mlog.AnsiText([mlog.green('YES'), ' ', mlog.cyan(self.version)])
+        return mlog.AnsiText(mlog.green('YES'), ' ', mlog.cyan(self.version))
 
     def get_compile_args(self) -> T.List[str]:
         if self.include_type == 'system':
@@ -128,7 +136,7 @@ class Dependency(HoldableObject):
     def get_all_compile_args(self) -> T.List[str]:
         """Get the compile arguments from this dependency and it's sub dependencies."""
         return list(itertools.chain(self.get_compile_args(),
-                                    *[d.get_all_compile_args() for d in self.ext_deps]))
+                                    *(d.get_all_compile_args() for d in self.ext_deps)))
 
     def get_link_args(self, language: T.Optional[str] = None, raw: bool = False) -> T.List[str]:
         if raw and self.raw_link_args is not None:
@@ -138,20 +146,16 @@ class Dependency(HoldableObject):
     def get_all_link_args(self) -> T.List[str]:
         """Get the link arguments from this dependency and it's sub dependencies."""
         return list(itertools.chain(self.get_link_args(),
-                                    *[d.get_all_link_args() for d in self.ext_deps]))
+                                    *(d.get_all_link_args() for d in self.ext_deps)))
 
     def found(self) -> bool:
         return self.is_found
 
-    def get_sources(self) -> T.List['FileOrString']:
+    def get_sources(self) -> T.List[T.Union['FileOrString', 'CustomTarget', 'StructuredSources']]:
         """Source files that need to be added to the target.
         As an example, gtest-all.cc when using GTest."""
         return self.sources
 
-    @staticmethod
-    def get_methods() -> T.List[DependencyMethods]:
-        return [DependencyMethods.AUTO]
-
     def get_name(self) -> str:
         return self.name
 
@@ -167,7 +171,9 @@ class Dependency(HoldableObject):
     def get_exe_args(self, compiler: 'Compiler') -> T.List[str]:
         return []
 
-    def get_pkgconfig_variable(self, variable_name: str, kwargs: T.Dict[str, T.Any]) -> str:
+    def get_pkgconfig_variable(self, variable_name: str,
+                               define_variable: 'ImmutableListProtocol[str]',
+                               default: T.Optional[str]) -> str:
         raise DependencyException(f'{self.name!r} is not a pkgconfig dependency')
 
     def get_configtool_variable(self, variable_name: str) -> str:
@@ -192,7 +198,7 @@ class Dependency(HoldableObject):
         raise RuntimeError('Unreachable code in partial_dependency called')
 
     def _add_sub_dependency(self, deplist: T.Iterable[T.Callable[[], 'Dependency']]) -> bool:
-        """Add an internal depdency from a list of possible dependencies.
+        """Add an internal dependency from a list of possible dependencies.
 
         This method is intended to make it easier to add additional
         dependencies to another dependency internally.
@@ -210,7 +216,7 @@ class Dependency(HoldableObject):
     def get_variable(self, *, cmake: T.Optional[str] = None, pkgconfig: T.Optional[str] = None,
                      configtool: T.Optional[str] = None, internal: T.Optional[str] = None,
                      default_value: T.Optional[str] = None,
-                     pkgconfig_define: T.Optional[T.List[str]] = None) -> T.Union[str, T.List[str]]:
+                     pkgconfig_define: T.Optional[T.List[str]] = None) -> str:
         if default_value is not None:
             return default_value
         raise DependencyException(f'No default provided for dependency {self!r}, which is not pkg-config, cmake, or config-tool based.')
@@ -221,10 +227,13 @@ class Dependency(HoldableObject):
         return new_dep
 
 class InternalDependency(Dependency):
-    def __init__(self, version: str, incdirs: T.List[str], compile_args: T.List[str],
-                 link_args: T.List[str], libraries: T.List['BuildTarget'],
-                 whole_libraries: T.List['BuildTarget'], sources: T.List['FileOrString'],
-                 ext_deps: T.List[Dependency], variables: T.Dict[str, T.Any]):
+    def __init__(self, version: str, incdirs: T.List['IncludeDirs'], compile_args: T.List[str],
+                 link_args: T.List[str],
+                 libraries: T.List[T.Union['BuildTarget', 'CustomTarget']],
+                 whole_libraries: T.List[T.Union['BuildTarget', 'CustomTarget']],
+                 sources: T.Sequence[T.Union['FileOrString', 'CustomTarget', StructuredSources]],
+                 ext_deps: T.List[Dependency], variables: T.Dict[str, str],
+                 d_module_versions: T.List[str], d_import_dirs: T.List['IncludeDirs']):
         super().__init__(DependencyTypeName('internal'), {})
         self.version = version
         self.is_found = True
@@ -233,9 +242,13 @@ class InternalDependency(Dependency):
         self.link_args = link_args
         self.libraries = libraries
         self.whole_libraries = whole_libraries
-        self.sources = sources
+        self.sources = list(sources)
         self.ext_deps = ext_deps
         self.variables = variables
+        if d_module_versions:
+            self.d_features['versions'] = d_module_versions
+        if d_import_dirs:
+            self.d_features['import_dirs'] = d_import_dirs
 
     def __deepcopy__(self, memo: T.Dict[int, 'InternalDependency']) -> 'InternalDependency':
         result = self.__class__.__new__(self.__class__)
@@ -258,7 +271,9 @@ class InternalDependency(Dependency):
             return True
         return any(d.is_built() for d in self.ext_deps)
 
-    def get_pkgconfig_variable(self, variable_name: str, kwargs: T.Dict[str, T.Any]) -> str:
+    def get_pkgconfig_variable(self, variable_name: str,
+                               define_variable: 'ImmutableListProtocol[str]',
+                               default: T.Optional[str]) -> str:
         raise DependencyException('Method "get_pkgconfig_variable()" is '
                                   'invalid for an internal dependency')
 
@@ -281,21 +296,15 @@ class InternalDependency(Dependency):
         return InternalDependency(
             self.version, final_includes, final_compile_args,
             final_link_args, final_libraries, final_whole_libraries,
-            final_sources, final_deps, self.variables)
+            final_sources, final_deps, self.variables, [], [])
 
     def get_variable(self, *, cmake: T.Optional[str] = None, pkgconfig: T.Optional[str] = None,
                      configtool: T.Optional[str] = None, internal: T.Optional[str] = None,
                      default_value: T.Optional[str] = None,
-                     pkgconfig_define: T.Optional[T.List[str]] = None) -> T.Union[str, T.List[str]]:
+                     pkgconfig_define: T.Optional[T.List[str]] = None) -> str:
         val = self.variables.get(internal, default_value)
         if val is not None:
-            # TODO: Try removing this assert by better typing self.variables
-            if isinstance(val, str):
-                return val
-            if isinstance(val, list):
-                for i in val:
-                    assert isinstance(i, str)
-                return val
+            return val
         raise DependencyException(f'Could not get an internal variable and no default provided for {self!r}')
 
     def generate_link_whole_dependency(self) -> Dependency:
@@ -324,6 +333,7 @@ class ExternalDependency(Dependency, HasNativeKwarg):
         self.required = kwargs.get('required', True)
         self.silent = kwargs.get('silent', False)
         self.static = kwargs.get('static', False)
+        self.libtype = LibType.STATIC if self.static else LibType.PREFER_SHARED
         if not isinstance(self.static, bool):
             raise DependencyException('Static keyword must be boolean')
         # Is this dependency to be run on the build platform?
@@ -397,10 +407,10 @@ class ExternalDependency(Dependency, HasNativeKwarg):
 
 
 class NotFoundDependency(Dependency):
-    def __init__(self, environment: 'Environment') -> None:
+    def __init__(self, name: str, environment: 'Environment') -> None:
         super().__init__(DependencyTypeName('not-found'), {})
         self.env = environment
-        self.name = 'not-found'
+        self.name = name
         self.is_found = False
 
     def get_partial_dependency(self, *, compile_args: bool = False,
@@ -493,14 +503,21 @@ def process_method_kw(possible: T.Iterable[DependencyMethods], kwargs: T.Dict[st
         raise DependencyException(f'method {method!r} is invalid')
     method = DependencyMethods(method)
 
+    # Raise FeatureNew where appropriate
+    if method is DependencyMethods.CONFIG_TOOL:
+        # FIXME: needs to get a handle on the subproject
+        # FeatureNew.single_use('Configuration method "config-tool"', '0.44.0')
+        pass
     # This sets per-tool config methods which are deprecated to to the new
     # generic CONFIG_TOOL value.
     if method in [DependencyMethods.SDLCONFIG, DependencyMethods.CUPSCONFIG,
                   DependencyMethods.PCAPCONFIG, DependencyMethods.LIBWMFCONFIG]:
-        FeatureDeprecated.single_use(f'Configuration method {method.value}', '0.44', 'Use "config-tool" instead.')
+        # FIXME: needs to get a handle on the subproject
+        #FeatureDeprecated.single_use(f'Configuration method {method.value}', '0.44', 'Use "config-tool" instead.')
         method = DependencyMethods.CONFIG_TOOL
     if method is DependencyMethods.QMAKE:
-        FeatureDeprecated.single_use(f'Configuration method "qmake"', '0.58', 'Use "config-tool" instead.')
+        # FIXME: needs to get a handle on the subproject
+        # FeatureDeprecated.single_use('Configuration method "qmake"', '0.58', 'Use "config-tool" instead.')
         method = DependencyMethods.CONFIG_TOOL
 
     # Set the detection method. If the method is set to auto, use any available method.
@@ -548,10 +565,6 @@ class SystemDependency(ExternalDependency):
         super().__init__(DependencyTypeName('system'), env, kwargs, language=language)
         self.name = name
 
-    @staticmethod
-    def get_methods() -> T.List[DependencyMethods]:
-        return [DependencyMethods.SYSTEM]
-
     def log_tried(self) -> str:
         return 'system'
 
@@ -565,9 +578,5 @@ class BuiltinDependency(ExternalDependency):
         super().__init__(DependencyTypeName('builtin'), env, kwargs, language=language)
         self.name = name
 
-    @staticmethod
-    def get_methods() -> T.List[DependencyMethods]:
-        return [DependencyMethods.BUILTIN]
-
     def log_tried(self) -> str:
         return 'builtin'
