@@ -44,7 +44,7 @@ from . import mlog
 from .coredata import major_versions_differ, MesonVersionMismatchException
 from .coredata import version as coredata_version
 from .mesonlib import (MesonException, OrderedSet, RealPathAction,
-                       get_wine_shortpath, join_args, split_args)
+                       get_wine_shortpath, join_args, split_args, setup_vsenv)
 from .mintro import get_infodir, load_info_file
 from .programs import ExternalProgram
 from .backend.backends import TestProtocol, TestSerialisation
@@ -477,8 +477,9 @@ class TestFileLogger(TestLogger):
 
 
 class ConsoleLogger(TestLogger):
-    SPINNER = "\U0001f311\U0001f312\U0001f313\U0001f314" + \
-              "\U0001f315\U0001f316\U0001f317\U0001f318"
+    ASCII_SPINNER = ['..', ':.', '.:']
+    SPINNER = ["\U0001f311", "\U0001f312", "\U0001f313", "\U0001f314",
+               "\U0001f315", "\U0001f316", "\U0001f317", "\U0001f318"]
 
     SCISSORS = "\u2700 "
     HLINE = "\u2015"
@@ -506,12 +507,14 @@ class ConsoleLogger(TestLogger):
         self.output_start = dashes(self.SCISSORS, self.HLINE, self.cols - 2)
         self.output_end = dashes('', self.HLINE, self.cols - 2)
         self.sub = self.RTRI
+        self.spinner = self.SPINNER
         try:
             self.output_start.encode(sys.stdout.encoding or 'ascii')
         except UnicodeEncodeError:
             self.output_start = dashes('8<', '-', self.cols - 2)
             self.output_end = dashes('', '-', self.cols - 2)
             self.sub = '| '
+            self.spinner = self.ASCII_SPINNER
 
     def flush(self) -> None:
         if self.should_erase_line:
@@ -536,8 +539,8 @@ class ConsoleLogger(TestLogger):
             count = '{}-{}/{}'.format(self.started_tests - len(self.running_tests) + 1,
                                       self.started_tests, self.test_count)
 
-        left = '[{}] {} '.format(count, self.SPINNER[self.spinner_index])
-        self.spinner_index = (self.spinner_index + 1) % len(self.SPINNER)
+        left = '[{}] {} '.format(count, self.spinner[self.spinner_index])
+        self.spinner_index = (self.spinner_index + 1) % len(self.spinner)
 
         right = '{spaces} {dur:{durlen}}'.format(
             spaces=' ' * TestResult.maxlen(),
@@ -597,17 +600,15 @@ class ConsoleLogger(TestLogger):
             self.progress_task = asyncio.ensure_future(report_progress())
 
     def start_test(self, harness: 'TestHarness', test: 'TestRun') -> None:
-        if harness.options.verbose and test.cmdline:
+        if test.verbose and test.cmdline:
             self.flush()
             print(harness.format(test, mlog.colorize_console(),
                                  max_left_width=self.max_left_width,
                                  right=test.res.get_text(mlog.colorize_console())))
             print(test.res.get_command_marker() + test.cmdline)
-            if test.needs_parsing:
-                pass
-            elif not test.is_parallel:
+            if test.direct_stdout:
                 print(self.output_start, flush=True)
-            else:
+            elif not test.needs_parsing:
                 print(flush=True)
 
         self.started_tests += 1
@@ -616,12 +617,12 @@ class ConsoleLogger(TestLogger):
         self.request_update()
 
     def shorten_log(self, harness: 'TestHarness', result: 'TestRun') -> str:
-        if not harness.options.verbose and not harness.options.print_errorlogs:
+        if not result.verbose and not harness.options.print_errorlogs:
             return ''
 
         log = result.get_log(mlog.colorize_console(),
                              stderr_only=result.needs_parsing)
-        if harness.options.verbose:
+        if result.verbose:
             return log
 
         lines = log.splitlines()
@@ -631,7 +632,7 @@ class ConsoleLogger(TestLogger):
             return str(mlog.bold('Listing only the last 100 lines from a long log.\n')) + '\n'.join(lines[-100:])
 
     def print_log(self, harness: 'TestHarness', result: 'TestRun') -> None:
-        if not harness.options.verbose:
+        if not result.verbose:
             cmdline = result.cmdline
             if not cmdline:
                 print(result.res.get_command_marker() + result.stdo)
@@ -645,7 +646,7 @@ class ConsoleLogger(TestLogger):
             print(self.output_end)
 
     def log_subtest(self, harness: 'TestHarness', test: 'TestRun', s: str, result: TestResult) -> None:
-        if harness.options.verbose or (harness.options.print_errorlogs and result.is_bad()):
+        if test.verbose or (harness.options.print_errorlogs and result.is_bad()):
             self.flush()
             print(harness.format(test, mlog.colorize_console(), max_left_width=self.max_left_width,
                                  prefix=self.sub,
@@ -656,22 +657,21 @@ class ConsoleLogger(TestLogger):
 
     def log(self, harness: 'TestHarness', result: 'TestRun') -> None:
         self.running_tests.remove(result)
-        if result.res is TestResult.TIMEOUT and harness.options.verbose:
+        if result.res is TestResult.TIMEOUT and result.verbose:
             self.flush()
             print(f'{result.name} time out (After {result.timeout} seconds)')
 
         if not harness.options.quiet or not result.res.is_ok():
             self.flush()
-            if harness.options.verbose and not result.is_parallel and result.cmdline:
-                if not result.needs_parsing:
-                    print(self.output_end)
+            if result.cmdline and result.direct_stdout:
+                print(self.output_end)
                 print(harness.format(result, mlog.colorize_console(), max_left_width=self.max_left_width))
             else:
                 print(harness.format(result, mlog.colorize_console(), max_left_width=self.max_left_width),
                       flush=True)
-                if harness.options.verbose or result.res.is_bad():
+                if result.verbose or result.res.is_bad():
                     self.print_log(harness, result)
-            if harness.options.verbose or result.res.is_bad():
+            if result.verbose or result.res.is_bad():
                 print(flush=True)
 
         self.request_update()
@@ -864,7 +864,7 @@ class TestRun:
         return super().__new__(TestRun.PROTOCOL_TO_CLASS[test.protocol])
 
     def __init__(self, test: TestSerialisation, test_env: T.Dict[str, str],
-                 name: str, timeout: T.Optional[int], is_parallel: bool):
+                 name: str, timeout: T.Optional[int], is_parallel: bool, verbose: bool):
         self.res = TestResult.PENDING
         self.test = test
         self._num = None       # type: T.Optional[int]
@@ -882,6 +882,7 @@ class TestRun:
         self.project = test.project_name
         self.junit = None      # type: T.Optional[et.ElementTree]
         self.is_parallel = is_parallel
+        self.verbose = verbose
 
     def start(self, cmd: T.List[str]) -> None:
         self.res = TestResult.RUNNING
@@ -895,6 +896,10 @@ class TestRun:
             self._num = TestRun.TEST_NUM
         return self._num
 
+    @property
+    def direct_stdout(self) -> bool:
+        return self.verbose and not self.is_parallel and not self.needs_parsing
+
     @property
     def detail(self) -> str:
         if self.res is TestResult.PENDING:
@@ -1332,33 +1337,37 @@ class SingleTestRunner:
             timeout = self.test.timeout * self.options.timeout_multiplier
 
         is_parallel = test.is_parallel and self.options.num_processes > 1 and not self.options.gdb
-        self.runobj = TestRun(test, env, name, timeout, is_parallel)
+        verbose = (test.verbose or self.options.verbose) and not self.options.quiet
+        self.runobj = TestRun(test, env, name, timeout, is_parallel, verbose)
 
         if self.options.gdb:
             self.console_mode = ConsoleUser.GDB
-        elif self.options.verbose and not is_parallel and not self.runobj.needs_parsing:
+        elif self.runobj.direct_stdout:
             self.console_mode = ConsoleUser.STDOUT
         else:
             self.console_mode = ConsoleUser.LOGGER
 
     def _get_test_cmd(self) -> T.Optional[T.List[str]]:
-        if self.test.fname[0].endswith('.jar'):
+        testentry = self.test.fname[0]
+        if self.options.no_rebuild and self.test.cmd_is_built and not os.path.isfile(testentry):
+            raise TestException(f'The test program {testentry!r} does not exist. Cannot run tests before building them.')
+        if testentry.endswith('.jar'):
             return ['java', '-jar'] + self.test.fname
-        elif not self.test.is_cross_built and run_with_mono(self.test.fname[0]):
+        elif not self.test.is_cross_built and run_with_mono(testentry):
             return ['mono'] + self.test.fname
-        elif self.test.cmd_is_built and self.test.is_cross_built and self.test.needs_exe_wrapper:
-            if self.test.exe_runner is None:
+        elif self.test.cmd_is_exe and self.test.is_cross_built and self.test.needs_exe_wrapper:
+            if self.test.exe_wrapper is None:
                 # Can not run test on cross compiled executable
                 # because there is no execute wrapper.
                 return None
-            elif self.test.cmd_is_built:
+            elif self.test.cmd_is_exe:
                 # If the command is not built (ie, its a python script),
                 # then we don't check for the exe-wrapper
-                if not self.test.exe_runner.found():
+                if not self.test.exe_wrapper.found():
                     msg = ('The exe_wrapper defined in the cross file {!r} was not '
                            'found. Please check the command and/or add it to PATH.')
-                    raise TestException(msg.format(self.test.exe_runner.name))
-                return self.test.exe_runner.get_command() + self.test.fname
+                    raise TestException(msg.format(self.test.exe_wrapper.name))
+                return self.test.exe_wrapper.get_command() + self.test.fname
         return self.test.fname
 
     def _get_cmd(self) -> T.Optional[T.List[str]]:
@@ -1572,8 +1581,8 @@ class TestHarness:
         test_env = test.env.get_env(env)
         env.update(test_env)
         if (test.is_cross_built and test.needs_exe_wrapper and
-                test.exe_runner and test.exe_runner.found()):
-            env['MESON_EXE_WRAPPER'] = join_args(test.exe_runner.get_command())
+                test.exe_wrapper and test.exe_wrapper.found()):
+            env['MESON_EXE_WRAPPER'] = join_args(test.exe_wrapper.get_command())
         return SingleTestRunner(test, env, name, options)
 
     def process_test_result(self, result: TestRun) -> None:
@@ -1797,7 +1806,7 @@ class TestHarness:
     def get_wrapper(options: argparse.Namespace) -> T.List[str]:
         wrap = []  # type: T.List[str]
         if options.gdb:
-            wrap = [options.gdb_path, '--quiet', '--nh']
+            wrap = [options.gdb_path, '--quiet']
             if options.repeat > 1:
                 wrap += ['-ex', 'run', '-ex', 'quit']
             # Signal the end of arguments to gdb
@@ -1998,6 +2007,9 @@ def run(options: argparse.Namespace) -> int:
             print(f'Could not find requested program: {check_bin!r}')
             return 1
 
+    b = build.load(options.wd)
+    setup_vsenv(b.need_vsenv)
+
     with TestHarness(options) as th:
         try:
             if options.list:
